# factor2.rb -- an example of Ruby/MathLink library.
# To run this program use the command-line below:
#      Unix:           ruby factor2.rb -linkname "math -mathlink"
#      Mac or Windows: ruby factor2.rb -linkname launch
# This example is ports of ${MATHLINK}/MathLinkExamples/factor2.c
# See also factor2.c.

require 'mathematica/mathlink'

include Mathematica
include MathLink

def error(link)
  if (link.error)
	printf STDERR, "Error detected by MathLink: %s.\n", link.error_message
  else
	printf STDERR, "Error detetected by this program.\n"
  end

  exit 3
end

read_and_print_expression = Proc.new do |link, indent|
  case link.get_next
  when MLTKSYM
	symbol = link.get_symbol
	print "#{symbol} "
	link.disown_symbol(symbol)
  when MLTKSTR
	str = link.get_string
	print %Q!"#{str}" !
	link.disown_string(str)
  when MLTKINT
	num = link.get_integer
	print "#{num} "
  when MLTKREAL
	real = link.get_real
	print "#{real} "
  when MLTKFUNC
	indent += 3
	printf "\n %*.*s", indent, indent, ''
	if ((len = link.get_arg_count) == 0)
	  error(link) 
	else
	  read_and_print_expression.call(link, indent)
	  print '['
	  1.upto(len) do |i|
		read_and_print_expression.call(link, indent)
		print ', ' if (i != len)
	  end
	  print ']'
	end
	indent -= 3
  when MLTKERROR
	error(link)
  else
	error(link)
  end
end

link = MathLink.new.open(ARGV.size, ARGV)

print 'Integer to factor: '
num = STDIN.gets

link.put_function('EvaluatePacket', 1)
link.put_function('FactorInteger', 1)
link.put_integer(num)
link.end_packet

while ((pkt = link.next_packet) && pkt != RETURNPKT)
  link.new_packet
  error(link) if (link.error)
end

read_and_print_expression.call(link, 0)
printf "\n"

link.put_function('Exit', 0)

